package com.flyfish.framework.query.chain;

import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.query.*;
import com.flyfish.framework.query.support.DomainFunction;
import lombok.RequiredArgsConstructor;

import java.util.Arrays;
import java.util.List;

/**
 * 默认的查询修改逻辑
 *
 * @author wangyu
 * <p>
 * 不修改原条件，而是在此基础上新增
 * 理论上查询不允许直接修改，而是通过该类做一个桥接
 * 也可以修改链的形式
 */
@RequiredArgsConstructor
public class DefaultQueryMutation implements QueryMutation {

    private final QueryChain<?, ?> previous;

    private final boolean within;

    /**
     * 以lambda的形式修改
     * 注意，如果之前用过lambda的形式，此处应该校验类型
     *
     * @param getter 方法引用
     * @return 结果
     */
    @Override
    public <T extends Domain> QueryCondition<LambdaQueryChain<T>> and(DomainFunction<T> getter) {
        LambdaQueryChain<T> newChain = new DefaultLambdaQueryChain<>();
        QueryCondition<LambdaQueryChain<T>> condition = newChain.and(getter);
        List<QueryChain<?, ?>> chains = Arrays.asList(previous, newChain);
        previous.and(chains);
        return condition;
    }

    /**
     * 以column的形式修改
     *
     * @param column 列名
     * @return 结果
     */
    @Override
    public QueryCondition<NamedQueryChain> and(String column) {
        return null;
    }

    /**
     * 直接以and拼接其他的所有条件
     *
     * @param chain 传入的查询
     * @return 结果
     */
    @Override
    public <T extends Domain> LambdaQueryChain<T> and(LambdaQueryChain<T> chain) {
        return null;
    }

    /**
     * 直接以and拼接其他的所有条件
     *
     * @param chain 传入的查询
     * @return 结果
     */
    @Override
    public NamedQueryChain and(NamedQueryChain chain) {
        return null;
    }

    /**
     * 以lambda的形式修改
     * 注意，如果之前用过lambda的形式，此处应该校验类型
     *
     * @param getter 方法引用
     * @return 结果
     */
    @Override
    public <T extends Domain> QueryCondition<LambdaQueryChain<T>> or(DomainFunction<T> getter) {
        return null;
    }

    /**
     * 以column的形式修改
     *
     * @param column 列名
     * @return 结果
     */
    @Override
    public QueryCondition<NamedQueryChain> or(String column) {
        return null;
    }

    /**
     * 直接以or拼接其他的所有条件
     *
     * @param chain 传入的查询
     * @return 结果
     */
    @Override
    public <T extends Domain> LambdaQueryChain<T> or(LambdaQueryChain<T> chain) {
        return null;
    }

    /**
     * 直接以or拼接其他的所有条件
     *
     * @param chain 传入的查询
     * @return 结果
     */
    @Override
    public NamedQueryChain or(NamedQueryChain chain) {
        return null;
    }
}
