package com.flyfish.framework.mongodb.query.impl;

import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.query.LambdaQueryChain;
import com.flyfish.framework.query.Queries;
import com.flyfish.framework.query.QueryChain;
import com.flyfish.framework.query.QueryCondition;
import com.flyfish.framework.query.support.DomainFunction;
import lombok.RequiredArgsConstructor;

import java.util.Collection;
import java.util.List;
import java.util.function.Supplier;

/**
 * mongodb实现的基于lambda的查询链
 *
 * @author wangyu
 */
public class MongoLambdaQueryChain<T extends Domain> extends MongoQueryDefinition implements LambdaQueryChain<T> {

    /**
     * 以且连接下一个字段
     *
     * @param column 列
     * @return 查询条件
     */
    @Override
    public QueryCondition<LambdaQueryChain<T>> and(DomainFunction<T> column) {
        return null;
    }

    /**
     * 直接拼接提供者，此处懒加载，最终build才会执行
     *
     * @param supplier 提供者
     * @return 结果
     */
    @Override
    public <V extends QueryChain<V, ?>> LambdaQueryChain<T> and(Supplier<QueryChain<V, ?>> supplier) {
        return null;
    }

    /**
     * 条件列表
     *
     * @param chain 多个条件们
     * @return 结果
     */
    @Override
    public LambdaQueryChain<T> and(LambdaQueryChain<T> chain) {
        return null;
    }

    /**
     * 多个嵌套子条件列表
     *
     * @param lambdaQueryChains 多条链
     * @param combinator        各个链条之间的连接方式
     * @return 结果
     */
    @Override
    public LambdaQueryChain<T> and(List<LambdaQueryChain<T>> lambdaQueryChains, Queries.Combinator combinator) {
        return null;
    }

    /**
     * 以或连接下一个字段
     *
     * @param column 列
     * @return 查询条件
     */
    @Override
    public QueryCondition<LambdaQueryChain<T>> or(DomainFunction<T> column) {
        return null;
    }

    /**
     * 以或连接条件列表
     *
     * @param chain 多个条件们
     * @return 结果
     */
    @Override
    public LambdaQueryChain<T> or(LambdaQueryChain<T> chain) {
        return null;
    }

    /**
     * 以或连接嵌套多个子条件列表
     *
     * @param lambdaQueryChains 多条链
     * @param combinator        各个链条之间的连接方式
     * @return 结果
     */
    @Override
    public LambdaQueryChain<T> or(List<LambdaQueryChain<T>> lambdaQueryChains, Queries.Combinator combinator) {
        return null;
    }

    /**
     * 直接拼接提供者，此处懒加载，最终build才会执行
     *
     * @param supplier 提供者
     * @return 结果
     */
    @Override
    public <V extends QueryChain<V, ?>> LambdaQueryChain<T> or(Supplier<QueryChain<V, ?>> supplier) {
        return null;
    }

    /**
     * 用于lambda的条件拼接
     */
    @RequiredArgsConstructor
    class LambdaQueryCondition implements QueryCondition<LambdaQueryChain<T>> {

        private final DomainFunction<T> field;

        /**
         * 相等判定
         *
         * @param value 值
         * @return 结果
         */
        @Override
        public LambdaQueryChain<T> eq(Object value) {
            criteria.and("").is(value);
            return MongoLambdaQueryChain.this;
        }

        /**
         * 判定某列的值中存在指定值，特指json array数据类型且子类型中带有id的场景
         * 用于兼容mongodb查询，mysql查询使用JSON_CONTAINS进行判定。
         * 当且仅当mongodb会拼接.$id，mysql一律匹配id字段
         * <p>
         * 等价于 .eq(value)
         *
         * @param value 值
         * @return 结果
         */
        @Override
        public LambdaQueryChain<T> hasId(Object value) {
            return null;
        }

        /**
         * 对于mongodb，自动处理。对于关系型数据库，代表json array中是否包含对应值，值仅支持基本数据类型
         * <p>
         * 等价于 .eq(value)
         *
         * @param value 基本数据类型的值
         * @return 结果
         */
        @Override
        public LambdaQueryChain<T> has(Object value) {
            return null;
        }

        /**
         * 不等判定
         *
         * @param value 值
         * @return 结果
         */
        @Override
        public LambdaQueryChain<T> ne(Object value) {
            return null;
        }

        /**
         * 值介于两者之间
         *
         * @param items 双值列表
         * @return 结果
         */
        @Override
        public LambdaQueryChain<T> between(List<?> items) {
            return null;
        }

        /**
         * 模糊匹配，这里是全模糊
         *
         * @param keyword 查询关键字
         * @return 结果
         */
        @Override
        public LambdaQueryChain<T> like(String keyword) {
            return null;
        }

        /**
         * 根据指定的方向进行模糊查询
         *
         * @param keyword   关键字
         * @param direction 方向，可以匹配开头和结尾
         * @return 结果
         */
        @Override
        public LambdaQueryChain<T> like(String keyword, Queries.Direction direction) {
            return null;
        }

        /**
         * 判定为空
         *
         * @return 结果
         */
        @Override
        public LambdaQueryChain<T> isNull() {
            return null;
        }

        /**
         * 包含在内
         *
         * @param list 集合
         * @return 结果
         */
        @Override
        public LambdaQueryChain<T> in(Collection<?> list) {
            return null;
        }

        /**
         * 包含在内
         * 注意，需要根据字段类型推断。
         * 如果是json数组，需要进行双向匹配
         *
         * @param values 值们
         * @return 结果
         */
        @Override
        public LambdaQueryChain<T> in(Object... values) {
            return null;
        }
    }
}
