package com.flyfish.framework.r2dbc.query.factory;

import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.query.LambdaQueryChain;
import com.flyfish.framework.query.spi.QueryChainFactory;
import com.flyfish.framework.r2dbc.query.impl.R2dbcLambdaQueryChain;

/**
 * mongo lambda 查询工厂
 *
 * @author wangyu
 */
public class R2dbcLambdaQueryFactory<T extends Domain> implements QueryChainFactory<LambdaQueryChain<T>> {

    /**
     * 生产适配器
     * 此接口为高度抽象接口，目的是兼容更多的查询链类型，便于后续扩展
     *
     * @return 生产结果
     */
    @Override
    public LambdaQueryChain<T> produce() {
        return new R2dbcLambdaQueryChain<>();
    }

    /**
     * 判定是否支持特定的实例类型
     *
     * @param targetType 工厂目标类型
     * @return 是否匹配
     */
    @Override
    public boolean supports(Class<?> targetType) {
        return targetType.isAssignableFrom(R2dbcLambdaQueryChain.class);
    }
}
